/*
 * Copyright 2000 by Jean-Claude Christophe <jch@oleane.net>
 *                                          <jch@bsdfr.org>
 *
 * Home page:  http://www.bsdfr.org/projects/tac_bsd/
 *
 * main.c  Core program.
 */

#include <sys/types.h>
#include <sys/socket.h> /* shutdown(2) */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "draft2.h"
#include "tac_bsd.h"
#include "conf.h"

/* in login.c */
extern void login();
extern char *stypeof(char *);
extern int  local_user(void);
extern void login_local(void);
extern void login_remote(void);

/* in misc.c */
extern void authenticate(void);

char *username; /* username we will use for authentication */
char *password; /* idem for password */
char *port;     /* current tty */
char *term;     /* our teminal info */

int try = 0, done = 0; /* some check for the main loop */

void
display_file(file)
    char *file;
{
    FILE *fp;
    char c[256];
    
    fp = fopen(file, "r"); /* we don't check, read_conf() already did it. */
    while (fgets(c, 256, fp))
	printf("%s", c);
}

void
close_quit()
{
    int desc;

    for (desc = getdtablesize(); desc > 2; desc--) {
	shutdown(desc, SHUT_RDWR);
	close(desc);
    }
    exit(-1);
}

int
main(argc, argv)
    int argc;
    char *argv[];
{
    if (!read_conf(CONF_FILE)) {
	perror(CONF_FILE);
	exit(-1);
    }
    port = ttyname(STDIN_FILENO); /* What is the current tty ? it will be our port.
				   * STDIN_FILENO = 0
				   */
    term = stypeof(port + 5);     /* +5 for remove the "/dev" */
    
    username = (char *)malloc(256);
    if (!username) { /* not enough memory */
	perror("username");
	exit(-1);
    }
    
    /* display motd if set */
    if (premotd) display_file(premotd);
    
    printf("User Access Verification\n\n"); /* cisco's style banner */

    while ((try != maxtry) && !done) {
	/* now, it's time to be serious.
	 * Our authentication can be resumed into 4 steps (we use TYPE_ASCII):
	 * 1) after established the connection, we send the START request
	 * 2) we are waiting for the REPLY
	 * 3) If the REPLY seems to be correct, we send the CONTINUE packet
	 * 4) The daemon reply by PASS or FAIL: the result we want.
	 */

	/* ask for username and password */
	authenticate();

	/* We take a look at the local password database only if passwd_care is set */
	if ((local_user() > 0) && passwd_care) {
	    login_local();
	} else
	    login_remote();
    } /* while(.. */

    if (try == maxtry) {
	if (motdfail) {
	    display_file(motdfail);
	    close_quit();
	}
    } else if (motdpass) { /* all is good, we accept the user to log into this machine */
	display_file(motdpass);
	login();
    }
    
    return(0);
}
